/*
 * Copyright (c) 2011 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <unistd.h>
#include <usb.h>
#include <ftdi.h>

#include "ftdi-emergency.h"

/**
 ** Init connection 
 **/
FE_STR *
fe_init(void)
{
	struct ftdi_eeprom ftdi_eeprom;
	FE_STR *fe;
	char *eeprom;

	fe = (FE_STR *)calloc(sizeof(FE_STR), 1);
	if (fe == NULL)
		return NULL;

	fe->ctx = ftdi_new();
	/* find the ftdi device */
	if (ftdi_usb_open_desc(fe->ctx, FE_USB_VENDOR, FE_USB_PRODUCT,
		NULL, FE_USB_SERIAL) < 0) {
		fprintf(stderr, "fti_usb_open: %s\n",
		    ftdi_get_error_string(fe->ctx));
		free(fe);
		return NULL;
	}
#if 0
	/* Read eeprom */
	eeprom = malloc(fe->ctx->eeprom_size);
	if (eeprom == NULL) {
		return NULL;
	}
	ftdi_read_eeprom(fe->ctx, eeprom);
	ftdi_eeprom_decode(&ftdi_eeprom, eeprom, fe->ctx->eeprom_size);
	printf("Vendor  0x%x\n", ftdi_eeprom.vendor_id);
	printf("Product 0x%x\n", ftdi_eeprom.product_id);
	printf("Self powered %d\n", ftdi_eeprom.self_powered);
	printf("Remote wakeup %d\n", ftdi_eeprom.remote_wakeup);
	printf("Chip Type %d\n", ftdi_eeprom.BM_type_chip);
	printf("IN is isochronous %d\n", ftdi_eeprom.in_is_isochronous);
	printf("OUT is isochronous %d\n", ftdi_eeprom.out_is_isochronous);
	printf("Manufacturer %s\n", ftdi_eeprom.manufacturer);
	printf("Product %s\n", ftdi_eeprom.product);
	printf("Serial %s\n", ftdi_eeprom.serial);
	printf("ROM size %d %d\n", ftdi_eeprom.size, fe->ctx->eeprom_size);
#endif

	ftdi_read_data_set_chunksize(fe->ctx, 16);
	fe->pins = 0;
	return fe;
}

/**
 ** Close connection
 **/
void
fe_end(FE_STR *fe)
{
	/* close ftdi device */
	ftdi_free(fe->ctx);
	free(fe);
}


/**
 ** Get data
 **/
int
fe_get_status(FE_STR *fe)
{
	int n;
	unsigned char data;

	/* read data from ftdi chip */
	n = ftdi_read_data(fe->ctx, &data, 1);
	if (n == 0)
		return 0;
	if (n == -666) {
		fprintf(stderr, "ftdi device not available\n");
		return -1;
	}
	if (n < 0)  {
		fprintf(stderr, "ftdi_read_pins: %s\n",
		    ftdi_get_error_string(fe->ctx));
		return -1;
	}
	data = data & 0x0f;
	fe->pins = data;
	return 1;
}
